use sqlx::SqlitePool;

use super::{sessions::Session, users::User};

pub struct Device {
    id: i64,
    user_id: i64,
    device_id: String,
    display_name: String,
}

impl Device {
    pub async fn create(
        conn: &SqlitePool,
        user: &User,
        device_id: &str,
        display_name: &str,
    ) -> anyhow::Result<Self> {
        let user_id = user.id();
        Ok(sqlx::query_as!(Self, "insert into devices(user_id, device_id, display_name) values(?, ?, ?) returning id, user_id, device_id, display_name", user_id, device_id, display_name).fetch_one(conn).await?)
    }

    pub async fn by_user(conn: &SqlitePool, user: &User) -> anyhow::Result<Self> {
        let user_id = user.id();
        Ok(sqlx::query_as!(
            Self,
            "select id, user_id, device_id, display_name from devices where user_id = ?",
            user_id
        )
        .fetch_one(conn)
        .await?)
    }

    pub async fn create_session(&self, conn: &SqlitePool) -> anyhow::Result<Session> {
        Ok(Session::create(conn, self, "random_session_id").await?)
    }

    /// Get the device's id.
    #[must_use]
    pub fn id(&self) -> i64 {
        self.id
    }

    /// Get the device's user id.
    #[must_use]
    pub fn user_id(&self) -> i64 {
        self.user_id
    }

    /// Get a reference to the device's device id.
    #[must_use]
    pub fn device_id(&self) -> &str {
        self.device_id.as_ref()
    }

    /// Get a reference to the device's display name.
    #[must_use]
    pub fn display_name(&self) -> &str {
        self.display_name.as_ref()
    }
}
