use sqlx::SqlitePool;

use crate::types::matrix_user_id::UserId;

pub struct User {
    id: i64,
    user_id: String,
    display_name: String,
    password: String,
}

impl User {
    pub async fn exists(conn: &SqlitePool, user_id: &UserId) -> anyhow::Result<bool> {
        Ok(
            sqlx::query!("select user_id from users where user_id = ?", user_id)
                .fetch_optional(conn)
                .await?
                .is_some(),
        )
    }

    pub async fn create(
        conn: &SqlitePool,
        user_id: &UserId,
        display_name: &str,
        password: &str,
    ) -> anyhow::Result<Self> {
        Ok(sqlx::query_as!(Self, "insert into users(user_id, display_name, password) values (?, ?, ?) returning id, user_id, display_name, password", user_id, display_name, password).fetch_one(conn).await?)
    }

    pub async fn by_user_id(conn: &SqlitePool, user_id: &UserId) -> anyhow::Result<Self> {
        Ok(sqlx::query_as!(
            Self,
            "select id, user_id, display_name, password from users where user_id = ?",
            user_id
        )
        .fetch_one(conn)
        .await?)
    }

    /// Get the user's id.
    #[must_use]
    pub fn id(&self) -> i64 {
        self.id
    }

    /// Get a reference to the user's user id.
    #[must_use]
    pub fn user_id(&self) -> &str {
        self.user_id.as_ref()
    }

    /// Get a reference to the user's password.
    #[must_use]
    pub fn password(&self) -> &str {
        self.password.as_ref()
    }
}
